﻿var caf = caf || {};
(function ($) {
  if (!$) {
    throw "caf/jquery library requires the jquery library included to the page!";
  }

  // caf CORE OVERRIDES /////////////////////////////////////////////////////

  caf.message._showMessage = function (message, title) {
    alert((title || "") + " " + message);

    return $.Deferred(function ($dfd) {
      $dfd.resolve();
    });
  };

  caf.message.confirm = function (message, titleOrCallback, callback) {
    if (titleOrCallback && !(typeof titleOrCallback == "string")) {
      callback = titleOrCallback;
    }

    var result = confirm(message);
    callback && callback(result);

    return $.Deferred(function ($dfd) {
      $dfd.resolve(result);
    });
  };

  caf.utils.isFunction = function (obj) {
    return $.isFunction(obj);
  };

  // JQUERY EXTENSIONS //////////////////////////////////////////////////////

  $.fn.findWithSelf = function (selector) {
    return this.filter(selector).add(this.find(selector));
  };

  // DOM ////////////////////////////////////////////////////////////////////

  caf.dom = caf.dom || {};

  caf.dom.onNodeAdded = function (callback) {
    caf.event.on("caf.dom.nodeAdded", callback);
  };

  caf.dom.onNodeRemoved = function (callback) {
    caf.event.on("caf.dom.nodeRemoved", callback);
  };

  var mutationObserverCallback = function (mutationsList) {
    for (var i = 0; i < mutationsList.length; i++) {
      var mutation = mutationsList[i];
      if (mutation.type === "childList") {
        if (mutation.addedNodes && mutation.removedNodes.length) {
          for (var k = 0; k < mutation.removedNodes.length; k++) {
            caf.event.trigger("caf.dom.nodeRemoved", {
              $el: $(mutation.removedNodes[k]),
            });
          }
        }

        if (mutation.addedNodes && mutation.addedNodes.length) {
          for (var j = 0; j < mutation.addedNodes.length; j++) {
            caf.event.trigger("caf.dom.nodeAdded", {
              $el: $(mutation.addedNodes[j]),
            });
          }
        }
      }
    }
  };

  new MutationObserver(mutationObserverCallback).observe($("body")[0], {
    subtree: true,
    childList: true,
  });

  // AJAX ///////////////////////////////////////////////////////////////////

  caf.ajax = function (userOptions) {
    userOptions = userOptions || {};

    var options = $.extend(true, {}, caf.ajax.defaultOpts, userOptions);

    options.success = undefined;
    options.error = undefined;

    return $.Deferred(function ($dfd) {
      $.ajax(options)
        .done(function (data, textStatus, jqXHR) {
          $dfd.resolve(data);
          userOptions.success && userOptions.success(data);
        })
        .fail(function (jqXHR) {
          if (jqXHR.getResponseHeader("_cafErrorFormat") === "true") {
            caf.ajax.handlecafErrorResponse(jqXHR, userOptions, $dfd);
          } else {
            caf.ajax.handleNoncafErrorResponse(jqXHR, userOptions, $dfd);
          }
        });
    });
  };

  $.extend(caf.ajax, {
    defaultOpts: {
      dataType: "json",
      type: "POST",
      contentType: "application/json",
      headers: {
        "X-Requested-With": "XMLHttpRequest",
      },
    },

    defaultError: {
      message: "An error has occurred!",
      details: "Error detail not sent by server.",
    },

    defaultError401: {
      message: "You are not authenticated!",
      details:
        "You should be authenticated (sign in) in order to perform this operation.",
    },

    defaultError403: {
      message: "You are not authorized!",
      details: "You are not allowed to perform this operation.",
    },

    defaultError404: {
      message: "Resource not found!",
      details: "The resource requested could not found on the server.",
    },

    logError: function (error) {
      caf.log.error(error);
    },

    showError: function (error) {
      if (error.details) {
        return caf.message.error(error.details, error.message);
      } else {
        return caf.message.error(
          error.message || caf.ajax.defaultError.message
        );
      }
    },

    handleTargetUrl: function (targetUrl) {
      if (!targetUrl) {
        location.href = caf.appPath;
      } else {
        location.href = targetUrl;
      }
    },

    handleErrorStatusCode: function (status) {
      switch (status) {
        case 401:
          caf.ajax.handleUnAuthorizedRequest(
            caf.ajax.showError(caf.ajax.defaultError401),
            caf.appPath
          );
          break;
        case 403:
          caf.ajax.showError(caf.ajax.defaultError403);
          break;
        case 404:
          caf.ajax.showError(caf.ajax.defaultError404);
          break;
        default:
          caf.ajax.showError(caf.ajax.defaultError);
          break;
      }
    },

    handleNoncafErrorResponse: function (jqXHR, userOptions, $dfd) {
      if (userOptions.cafHandleError !== false) {
        caf.ajax.handleErrorStatusCode(jqXHR.status);
      }

      $dfd.reject.apply(this, arguments);
      userOptions.error && userOptions.error.apply(this, arguments);
    },

    handlecafErrorResponse: function (jqXHR, userOptions, $dfd) {
      var messagePromise = null;

      if (userOptions.cafHandleError !== false) {
        messagePromise = caf.ajax.showError(jqXHR.responseJSON.error);
      }

      caf.ajax.logError(jqXHR.responseJSON.error);

      $dfd && $dfd.reject(jqXHR.responseJSON.error, jqXHR);
      userOptions.error && userOptions.error(jqXHR.responseJSON.error, jqXHR);

      if (jqXHR.status === 401 && userOptions.cafHandleError !== false) {
        caf.ajax.handleUnAuthorizedRequest(messagePromise);
      }
    },

    handleUnAuthorizedRequest: function (messagePromise, targetUrl) {
      if (messagePromise) {
        messagePromise.done(function () {
          caf.ajax.handleTargetUrl(targetUrl);
        });
      } else {
        caf.ajax.handleTargetUrl(targetUrl);
      }
    },

    blockUI: function (options) {
      if (options.blockUI) {
        if (options.blockUI === true) {
          //block whole page
          caf.ui.setBusy();
        } else {
          //block an element
          caf.ui.setBusy(options.blockUI);
        }
      }
    },

    unblockUI: function (options) {
      if (options.blockUI) {
        if (options.blockUI === true) {
          //unblock whole page
          caf.ui.clearBusy();
        } else {
          //unblock an element
          caf.ui.clearBusy(options.blockUI);
        }
      }
    },

    ajaxSendHandler: function (event, request, settings) {
      var token = caf.security.antiForgery.getToken();
      if (!token) {
        return;
      }

      if (
        !settings.headers ||
        settings.headers[caf.security.antiForgery.tokenHeaderName] === undefined
      ) {
        request.setRequestHeader(
          caf.security.antiForgery.tokenHeaderName,
          token
        );
      }
    },
  });

  $(document).ajaxSend(function (event, request, settings) {
    return caf.ajax.ajaxSendHandler(event, request, settings);
  });

  caf.event.on("caf.configurationInitialized", function () {
    var l = caf.localization.getResource("cafUi");

    caf.ajax.defaultError.message = l("DefaultErrorMessage");
    caf.ajax.defaultError.details = l("DefaultErrorMessageDetail");
    caf.ajax.defaultError401.message = l("DefaultErrorMessage401");
    caf.ajax.defaultError401.details = l("DefaultErrorMessage401Detail");
    caf.ajax.defaultError403.message = l("DefaultErrorMessage403");
    caf.ajax.defaultError403.details = l("DefaultErrorMessage403Detail");
    caf.ajax.defaultError404.message = l("DefaultErrorMessage404");
    caf.ajax.defaultError404.details = l("DefaultErrorMessage404Detail");
  });

  // RESOURCE LOADER ////////////////////////////////////////////////////////

  /* UrlStates enum */
  var UrlStates = {
    LOADING: "LOADING",
    LOADED: "LOADED",
    FAILED: "FAILED",
  };

  /* UrlInfo class */
  function UrlInfo(url) {
    this.url = url;
    this.state = UrlStates.LOADING;
    this.loadCallbacks = [];
    this.failCallbacks = [];
  }

  UrlInfo.prototype.succeed = function () {
    this.state = UrlStates.LOADED;
    for (var i = 0; i < this.loadCallbacks.length; i++) {
      this.loadCallbacks[i]();
    }
  };

  UrlInfo.prototype.failed = function () {
    this.state = UrlStates.FAILED;
    for (var i = 0; i < this.failCallbacks.length; i++) {
      this.failCallbacks[i]();
    }
  };

  UrlInfo.prototype.handleCallbacks = function (loadCallback, failCallback) {
    switch (this.state) {
      case UrlStates.LOADED:
        loadCallback && loadCallback();
        break;
      case UrlStates.FAILED:
        failCallback && failCallback();
        break;
      case UrlStates.LOADING:
        this.addCallbacks(loadCallback, failCallback);
        break;
    }
  };

  UrlInfo.prototype.addCallbacks = function (loadCallback, failCallback) {
    loadCallback && this.loadCallbacks.push(loadCallback);
    failCallback && this.failCallbacks.push(failCallback);
  };

  /* ResourceLoader API */

  caf.ResourceLoader = (function () {
    var _urlInfos = {};

    function getCacheKey(url) {
      return url;
    }

    function appendTimeToUrl(url) {
      if (url.indexOf("?") < 0) {
        url += "?";
      } else {
        url += "&";
      }

      url += "_=" + new Date().getTime();

      return url;
    }

    var _loadFromUrl = function (
      url,
      loadCallback,
      failCallback,
      serverLoader
    ) {
      var cacheKey = getCacheKey(url);

      var urlInfo = _urlInfos[cacheKey];

      if (urlInfo) {
        urlInfo.handleCallbacks(loadCallback, failCallback);
        return;
      }

      _urlInfos[cacheKey] = urlInfo = new UrlInfo(url);
      urlInfo.addCallbacks(loadCallback, failCallback);

      serverLoader(urlInfo);
    };

    var _loadScript = function (url, loadCallback, failCallback) {
      _loadFromUrl(url, loadCallback, failCallback, function (urlInfo) {
        $.get({
          url: url,
          dataType: "text",
        })
          .done(function (script) {
            $.globalEval(script);
            urlInfo.succeed();
          })
          .fail(function () {
            urlInfo.failed();
          });
      });
    };

    var _loadStyle = function (url) {
      _loadFromUrl(url, undefined, undefined, function (urlInfo) {
        $("<link/>", {
          rel: "stylesheet",
          type: "text/css",
          href: appendTimeToUrl(url),
        }).appendTo("head");
      });
    };

    return {
      loadScript: _loadScript,
      loadStyle: _loadStyle,
    };
  })();
})(jQuery);
